# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
#
# $Id: interfaces.py 490 2006-10-08 02:02:54Z seang $
# =============================================================================

try:
    from zope.interface import Interface, Attribute
except ImportError:
    class Interface:
        pass
    class Attribute:
        def __init__(self, *args):
            pass

# =============================================================================
# Web Service Interfaces
# =============================================================================

class IOWSDescriptor(Interface):
    """Describes an OGC Web Service."""
    
    url = Attribute("Base URL for requests (string)")

    version = Attribute("Protocol version, such as '1.1.1' (string)")

class IOWSMapRequest(Interface):
    """Encapsulate an WMS getmap request."""

    ows_format = Attribute("Request format, like 'image/jpeg' (string)")

    ows_srs = Attribute("Request SRS, like 'EPSG:4326' (string)")


class IMapRenderer(Interface):
    """Map rendering interface.  Make a map from a set of layers."""

    def render(layering, view, format, size, bgcolor, transparent):
        """Makes map from layers, returning an object specified by the
        format and size parameters.
        
        Arguments
        ---------
        layering : iterable
            A list of (Layer, Style) tuples in "painting" order
            
        view : View
            spatial extent and reference system of output map
            
        format : string
            A named format specific to the implementor
            
        size : iterable
            image type implementors should require a width, height
            pair specifying the pixel dimensions of the output map
            
        bgcolor : string
            background color for image type maps
            
        transparent : boolean
            transparent background or not for image type maps
        """

    def draw(rule, format, size=[16, 16], bgcolor='#FFFFFF', transparent=False):
        """Makes an icon-sized image of a symbolizer.

        See the render method for explanation of arguments
        """

class IViewport(Interface):
    pass


