# $Id: testdata.py,v 1.23 2005/05/19 17:15:06 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

import os
import sys
import unittest
#import threading

try:
    import pkg_resources
    pkg_resources.require('PCL-Core')
except ImportError:
    pass

from zope.interface.verify import verifyClass
from cartography.data.interfaces import IWMSStore, IWFSStore
from cartography.data.ows import WMSRasterStore, WMSRasterSource, \
    WFSFeatureStore, WFSFeatureSource

# ============================================================================
# Simple tests of WMS store and source
# ============================================================================

class WMSStoreTest(unittest.TestCase):
    def setUp(self):
        url = 'http://wms.jpl.nasa.gov/wms.cgi'
        self.store = WMSRasterStore(url)

    def test_interface(self):
        self.failUnless(verifyClass(IWMSStore, WMSRasterStore))

    def test_store(self):
        self.assertEqual(self.store.typenames(),
            ['global_mosaic', 'global_mosaic_base', 'us_landsat_wgs84',
             'srtm_mag', 'daily_terra_721', 'daily_aqua_721',
             'daily_terra_ndvi', 'daily_aqua_ndvi', 'daily_terra',
             'daily_aqua', 'BMNG', 'modis', 'huemapped_srtm', 'srtmplus',
             'worldwind_dem', 'us_ned', 'us_elevation', 'us_colordem'])
        b = self.store.bounds('global_mosaic')
        self.assertEqual(b.totuple(), (-180.0, -60.0, 180.0, 84.0))
        self.assertEqual(b.srs.tostring(), '+init=epsg:4326')
        self.assertEqual(self.store.info('global_mosaic'),
            {'typename': 'global_mosaic', 'srs': ['EPSG:4326', 'AUTO:42003']})

    def test_source0(self):
        source = self.store.source('global_mosaic/visual')
        assert isinstance(source, WMSRasterSource)

class WFSStoreTest(unittest.TestCase):

    def setUp(self):
        url = 'http://sigma.openplans.org:8080/geoserver/wfs?'
        self.store = WFSFeatureStore(url)

    def test_interface(self):
        self.failUnless(verifyClass(IWFSStore, WFSFeatureStore))

    def test_store(self):
        self.assertEqual(self.store.typenames(), ['topp:water_polygon', 'topp:countries', 'topp:tasmania_state_boundaries', 'topp:poly_county', 'topp:major_roads', 'topp:water_shorelines', 'topp:states', 'topp:gnis_pop', 'topp:tasmania_roads', 'topp:pip', 'topp:tasmania_water_bodies', 'topp:roads', 'topp:gnis', 'topp:tasmania_cities', 'topp:poly_landmarks'])
        b = self.store.bounds('topp:roads')
        self.assertEqual(b.totuple(), (-179.23109436035199, -89.999899999999997, 179.9999, 89.999899999999997))
        self.assertEqual(b.srs.tostring(), '+init=epsg:4326')
        self.assertEqual(self.store.info('topp:roads'), {
                                                'typename': 'topp:roads',
                                                'srs': ['EPSG:4326'],
                                                'title': 'roads_Type'
                                                })


# =============================================================================
if __name__ == '__main__':
    unittest.main()

