# =============================================================================
# Python Cartographic Library. Copyright (C) 2005 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

import unittest

import pkg_resources
pkg_resources.require('PCL-Core')

from cartography.proj import SpatialReference
from cartography.proj.transform import transform_points
from cartography.proj import ProjTransform


class LowLevelTransformTest(unittest.TestCase):
    def test_lambert_wgs84(self):
        lambert = '+init=epsg:27582'
        wgs84 = '+init=epsg:4326'
        results = transform_points(lambert, wgs84, [(0.0, 47.0)])
        self.assertAlmostEqual(results[0][0], 600059.9936, 3), results[0]
        self.assertAlmostEqual(results[0][1], 2222238.6295, 3), results[0]
    def test_wgs_lambert(self):
        lambert = '+init=epsg:27582'
        wgs84 = '+init=epsg:4326'
        results = transform_points(wgs84, lambert, [(600059.9936,2222238.6295)])
        self.assertAlmostEqual(results[0][0], 0.0, 3), results[0]
        self.assertAlmostEqual(results[0][1], 47.0, 3), results[0]

class TransformTest(unittest.TestCase):
    def test_lambert_wgs84(self):
        lambert = SpatialReference(epsg=27582)
        wgs84 = SpatialReference(epsg=4326)
        t = ProjTransform(lambert, wgs84)
        results = t.transform([(0.0, 47.0)])
        self.assertAlmostEqual(results[0][0], 600059.9936, 3), results[0]
        self.assertAlmostEqual(results[0][1], 2222238.6295, 3), results[0]
    def test_wgs_lambert(self):
        lambert = SpatialReference(epsg=27582)
        wgs84 = SpatialReference(epsg=4326)
        t = ProjTransform(wgs84, lambert)
        results = t.transform([(600059.9936, 2222238.6295)])
        self.assertAlmostEqual(results[0][0], 0.0, 3), results[0]
        self.assertAlmostEqual(results[0][1], 47.0, 3), results[0]
    def test_utm_wgs(self):
        utm13 = SpatialReference('+proj=utm +zone=13 +ellps=GRS80 +datum=NAD83 +units=m +no_defs')
        wgs84 = SpatialReference(epsg=4326)
        t = ProjTransform(utm13, wgs84)
        results = t.transform([(-105.0, 40.0), (-106.0, 39.0)])
        self.assertAlmostEqual(results[0][0], 500000.0000, 3), results[0]
        self.assertAlmostEqual(results[0][1], 4427757.2186, 3), results[0]
        self.assertAlmostEqual(results[1][0], 413407.3219, 3), results[0]
        self.assertAlmostEqual(results[1][1], 4317252.1645, 3), results[0]
        
  
# =============================================================================
if __name__ == '__main__':
    unittest.main()

