# $Id: timequery.py,v 1.2 2005/02/10 16:10:07 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

# =============================================================================
# This script is for racing the PCL filtered feature access vs the mapscript
# query API.
#
# PCL's API is much more powerful, but we don't want to fall very far behind
# mapscript in performance.
# =============================================================================

import os
import sys
import timeit
import unittest
import mapscript

sys.path.insert(0, '../../..')
import cartography

from cartography import data, spatial
from cartography.filter import PythonFilter

TESTPATH = '/home/sean/projects/ms_44/mapserver/tests/'
TESTMAPFILE = os.path.join(TESTPATH, 'test.map')

mapobj = mapscript.mapObj(TESTMAPFILE)
l = mapobj.getLayer(1)

path = os.path.join(TESTPATH, l.data + '.shp')
store = data.DiskFeatureStore(path)

# =========================================================================
# Test 1A: layerObj.queryByRect()

r = mapobj.extent
print "Test 1A: mapscript.layerObj.queryByRect()"
s = """\
l.template = 'foo'
l.queryByRect(mapobj, r)
results = l.getResults()
l.open()
for i in range(results.numresults):
    result = results.getResult(i)
    shape = l.getFeature(result.shapeindex, result.tileindex)
    atts = [shape.getValue(j) for j in range(l.numitems)]
l.close()
"""
t = timeit.Timer(stmt=s, setup='from __main__ import mapobj, l, r')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)


# =========================================================================
# Test 1B: data.DiskFeatureStore.iterfeatures()

bbox = '%f,%f,%f,%f' % (r.minx, r.miny, r.maxx, r.maxy)
print "Test 1B: data.DiskFeatureStore.iterfeatures()"
s = """\
query = data.Query(filter=PythonFilter('g.BBOX(%s)' % (bbox)))
for g, a in store.iterfeatures(query):
    pass
"""
t = timeit.Timer(stmt=s, setup='from __main__ import data, PythonFilter, store, bbox')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)


# =========================================================================
# Test 2A: layerObj.queryByAttributes()

print "Test 2A: mapscript.layerObj.queryByAttributes()"
s = """\
l.template = 'foo'
l.queryByAttributes(mapobj, 'FNAME', 'A Polygon', 1)
results = l.getResults()
l.open()
for i in range(results.numresults):
    result = results.getResult(i)
    shape = l.getFeature(result.shapeindex, result.tileindex)
    atts = [shape.getValue(j) for j in range(l.numitems)]
l.close()
"""
t = timeit.Timer(stmt=s, setup='from __main__ import mapobj, l, r')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)


# =========================================================================
# Test 2B: data.DiskFeatureStore.iterfeatures()

print "Test 2B: data.DiskFeatureStore.iterfeatures()"
s = """\
query = data.Query(filter=PythonFilter('f.FNAME == "A Polygon"'))
for g, a in store.iterfeatures(query):
    pass
"""
t = timeit.Timer(stmt=s, setup='from __main__ import data, PythonFilter, store')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)


# =========================================================================
# Test 3A: layerObj.queryByPoint()

p = mapscript.pointObj(0.0, 51.5)
print "Test 3A: mapscript.layerObj.queryByPoint()"
s = """\
l.template = 'foo'
l.queryByPoint(mapobj, p, 1, -1.0)
results = l.getResults()
l.open()
for i in range(results.numresults):
    result = results.getResult(i)
    shape = l.getFeature(result.shapeindex, result.tileindex)
    atts = [shape.getValue(j) for j in range(l.numitems)]
l.close()
"""
t = timeit.Timer(stmt=s, setup='from __main__ import mapobj, l, p')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)


# =========================================================================
# Test 3B: data.DiskFeatureStore.iterfeatures()

p = spatial.Point(0.0, 51.5)
print "Test 3B: data.DiskFeatureStore.iterfeatures()"
s = """\
query = data.Query(filter=PythonFilter('g.intersects(p)'), p=p)
for g, a in store.iterfeatures(query):
    pass
"""
t = timeit.Timer(stmt=s, setup='from __main__ import data, PythonFilter, store, p')
print "%.2f usec/pass" % (1000000 * t.timeit(number=100)/100)



