
import sys
import timeit

# Compare PCL PostGIS feature access to ogr.py

sys.path.append('/home/sean/staging')
import pkg_resources
pkg_resources.require('PCL-Data')
pkg_resources.require('PCL-Spatial')

from cartography.data import postgis
from cartography.data.filter import SQLFilter, Query
from cartography.spatial import geometry
from cartography.spatial.envelope import Envelope

DSN = 'dbname=pcl_test user=postgres'

# PCL-Data
s = """
filter = SQLFilter("the_geom && SetSRID('BOX3D(-10 45, 10 60)'::box3d, 4269) AND fips_cntry = 'UK'")
query = Query(filter=filter)
store = postgis.PGFeatureStore(DSN)
source = store.source('postgis.world_borders', query)
for feature in source():
    id = feature.id
"""
t = timeit.Timer(stmt=s, setup='from __main__ import DSN, SQLFilter, Query, postgis')
print "PCL, all UK Features (48)"
print "%.2f usec/pass" % (1000000 * t.timeit(number=10)/10)
print

# OGR

sys.path.append('/home/sean/lib/python2.3/site-packages')
import ogr

s = """
filter_geom = ogr.CreateGeometryFromWkt('POLYGON((-10 45, -10 60, 10 60, 10 45, -10 45))')
source = ogr.Open("PG:%s" % (DSN))
layer = source.GetLayerByName('world_borders')
layer.ResetReading()
layer.SetSpatialFilter(filter_geom)
layer.SetAttributeFilter("fips_cntry = 'UK'")
while 1:
    feature = layer.GetNextFeature()
    if not feature:
        break
    id = feature.GetFID()
    feature.Destroy()
source.Destroy()
"""
print "OGR, all UK Features (48)"
t = timeit.Timer(stmt=s, setup='from __main__ import DSN, ogr')
print "%.2f usec/pass" % (1000000 * t.timeit(number=10)/10)

