# $Id: testrendering.py,v 1.3 2005/04/20 19:49:22 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

# pgrendering.py
#
# holding this out of our standard test suites for the present since it 
# requires a specific fixture

import os
import unittest
import mapscript

from framework import TESTPATH, FONTSET
from framework import cartography

from cartography import mapping, data, spatial, rendering, query
from cartography.referencing.srs import SpatialReference

class MapServerRendererTestCase(unittest.TestCase):
    """testing cartography.mapping.MapServerRenderer"""

    def setUp(self):

        # --------------------------------------------------------------------
        # borders polygon layer

        self.srs = SpatialReference(epsg=4326)
        self.dsn = 'host=lenny dbname=pcldev user=postgres'
        self.p_store = data.PGFeatureStore(self.dsn, 'world_borders.the_geom',
                                           srs=self.srs)
        self.p_layer = mapping.Layer(self.p_store, 'polygon')

        # a style for the polygon layer with two rules
        symb = mapping.PolygonSymbolizer(fill={'fill': '#FF8080'})
        text = mapping.TextSymbolizer('fips_cntry', {'font-family': 'VeraBd'})
        text2 = mapping.TextSymbolizer('fips_cntry', {'font-family': 'Vera', 
                                                 'font-size': 6.0})
        self.p_style = mapping.Style('p_style')

        # this rule is aimed at one particular feature
        f1 = query.PythonFilter('str(f.fips_cntry) == "UK"')
        self.p_style.rules.append(mapping.Rule('named', symb, f1))
        
        # for same feature with a Text Symbolizer
        self.p_style.rules.append(mapping.Rule('named', text, f1))
         
        # this rule, with no filter is the catch-all 
        
        self.p_style.rules.append(mapping.Rule('catch-all', 
                                               mapping.PolygonSymbolizer()))
        self.p_style.rules.append(mapping.Rule('catch-all', text2))

        self.mapper = rendering.MapRenderer('MAPSERVER', fontset=FONTSET,
                                          incoming='/var/www-tmp/')

    # tests

    def testPgLayerMap(self):
        """make a map image from a single layer using a defined style"""
        # the canvas
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
        
        image = self.mapper.render([(self.p_layer, self.p_style)], view, 
                                   'image/png', size, transparent=1)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testPgLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()


# ===========================================================================
if __name__ == '__main__':
    unittest.main()

