# $Id: testmsbridge.py,v 1.1 2005/01/27 04:53:51 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

import unittest

import pkg_resources
pkg_resources.require('PCL-Core')
pkg_resources.require('PCL-MapServer')

from cartography.geometry import Geometry
from cartography.engine.mapserver import msbridge
import mapscript

# ===========================================================================
# Tests begin now

class PointTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('POINT (-1 -1)')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_POINT)
        self.assertEqual(shape.numlines, 1)
    
class MultiPointTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('MULTIPOINT (-1 -1,-1 1,1 1,1 -1,-1 -1)')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_POINT)
        self.assertEqual(shape.numlines, 5)
     
class LineTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('LINESTRING (-1 -1,-1 1,1 1,1 -1,-1 -1)')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_LINE)
        self.assertEqual(shape.numlines, 1)
    
class MultiLineTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('MULTILINESTRING ((-1 -1,-1 1,1 1,1 -1,-1 -1))')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_LINE)
        self.assertEqual(shape.numlines, 1)
     
class PolygonTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('POLYGON ((-1 -1,-1 1,1 1,1 -1,-1 -1))')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_POLYGON)
        self.assertEqual(shape.numlines, 1)
      
class MultiPolygonTest(unittest.TestCase):
    def test_convert(self):
        geom = Geometry.fromWKT('MULTIPOLYGON (((-1 -1,-1 1,1 1,1 -1,-1 -1)))')
        shape = msbridge.shapeObj(0, geom)
        self.assertEqual(shape.type, mapscript.MS_SHAPE_POLYGON)
        self.assertEqual(shape.numlines, 1)
        
        
# ===========================================================================
if __name__ == '__main__':
    unittest.main()
