# $Id: testrendering.py,v 1.3 2005/04/20 19:49:22 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

import os
import unittest
import mapscript

from framework import TESTPATH, FONTSET
from framework import cartography

from cartography import mapping, spatial, rendering, query
from cartography.data import disk, memory, ows
from cartography.referencing.srs import SpatialReference

DISKSTORE = os.path.abspath(os.path.join(TESTPATH, 'diskstore.ovf'))

from MapServerRenderer import MapServerRendererTestCase

#(unittest.TestCase):
#    """testing cartography.mapping.MapServerRenderer"""
#    def setUp(self):
#        self.ul = spatial.Point(-10.0, 10.0)
#        self.lr = spatial.Point(10.0, -10.0)
#        self.srs = SpatialReference(epsg=4326)
#        self.view = mapping.View(self.srs, self.ul, self.lr)
#        
#        # store
#        self.path = DISKSTORE
#        self.d_store = disk.DiskFeatureStore(self.path)
#        # layers
#        self.x_layer = mapping.Layer(self.d_store, 'point')
#        self.l_layer = mapping.Layer(self.d_store, 'line')
#        self.p_layer = mapping.Layer(self.d_store, 'polygon')
#        
#        # -------------------------------------------------------------------
#        # Intrinsic Style
#        # -------------------------------------------------------------------
#        # a style for the polygon layer with two rules
#        symb = mapping.PolygonSymbolizer(fill={'fill': '#FF8080'})
#        text = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
#        text2 = mapping.TextSymbolizer('FNAME', {'font-family': 'Vera', 
#                                                 'font-size': 6.0}) 
#        self.p_style = mapping.Style('p_style')
#        # this rule is aimed at one particular feature
#        f1 = query.PythonFilter('str(f.FNAME) == "A Polygon"')
#        self.p_style.rules.append(mapping.Rule('named', symb, f1))
#        # for same feature with a Text Symbolizer
#        self.p_style.rules.append(mapping.Rule('named', text, f1))
#        # this rule, with no filter is the catch-all 
#        self.p_style.rules.append(mapping.Rule('catch-all', 
#                                               mapping.PolygonSymbolizer()))
#        self.p_style.rules.append(mapping.Rule('catch-all', text2))
#
#        # ------------------------------------------------------------------
#        # Extrinsic Style -- Rules are evaluated in Python, not by the
#        # mapping engine
#        self.pe_style = mapping.ExtrinsicStyle('pe_style')
#        # this rule is aimed at one particular feature
#        f2 = query.PythonFilter('f.FNAME.find("A P") == 0')
#        f2b = query.PythonFilter('f.FNAME.find("A P") != 0')
#        self.pe_style.rules.append(mapping.Rule('named', text, f2))
#        self.pe_style.rules.append(mapping.Rule('named', symb, f2))
#        # this rule, with no filter is the catch-all 
#        self.pe_style.rules.append(mapping.Rule('catch-all', text2, f2b))
#        self.pe_style.rules.append(mapping.Rule('catch-all', 
#                                               mapping.PolygonSymbolizer(), f2b))
#        self.pe_style.rules.append(mapping.Rule('false', 
#                                               mapping.PolygonSymbolizer(),
#                                               query.PythonFilter('False')))
#
#        # --------------------------------------------------------------------
#        # inline data layer
#        geoms = [spatial.Polygon(wkt='POLYGON ((-1.0 50.5, -0.5 51.2, 0.3 50.9, -1 50.5))'),
#                 spatial.Polygon(wkt='POLYGON((-0.7 50.3, 0.1 51.0, 0.6 50.1, -0.7 50.3))')]
#        atts = [('A Polygon',), ('B Polygon',)]
#        source = memory.MemoryFeatureSource('inline', [['FNAME', 'string']], self.srs)
#        for g,a in zip(geoms, atts):
#            source.add(g,a)
#        self.w_store = memory.MemoryFeatureStore()
#        self.w_store['inline'] = source
#        self.w_layer = mapping.Layer(self.w_store, 'inline')
#
#        # --------------------------------------------------------------------
#        # raster layer
#
#        self.r_path = os.path.join(TESTPATH, 'raster.vrt')
#        self.r_store = disk.DiskRasterStore(self.r_path)
#        self.r_layer = mapping.Layer(self.r_store, 'raster')
#
#        # --------------------------------------------------------------------
#        # WMS raster layer
#
#        self.wms_url = "http://wms.jpl.nasa.gov/wms.cgi?"
#        self.wms_store = ows.WMSRasterStore(self.wms_url, '1.1.1',
#                                            incoming='/var/www-tmp/')
#        self.wms_layer = mapping.Layer(self.wms_store,
#                                       'global_mosaic_base/visual')
#        # adapt
#        self.wms_layer.ows_srs = 'EPSG:4326'
#        self.wms_layer.ows_format = 'image/jpeg'
#
#        # WFS
#        u = 'http://zcologia.com:9001/mapserver/members/features.rpy'
#        self.wfs_store = ows.WFSFeatureStore(u, '1.0.0',
#                                             incoming='/var/www-tmp/')
#        self.wfs_layer = mapping.Layer(self.wfs_store, 'users')
#        self.wfs_layer.ows_srs = 'EPSG:4326'
#
#        # Point symbolization
#        circle = mapping.Mark('square')
#        fn = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
#        wfs_sym = mapping.PointSymbolizer(10, mark=circle)
#        # Text symbolization
#        wfs_text = mapping.TextSymbolizer('zco:fullname',{'font-family': 'VeraBd'})
#        self.wfs_style = mapping.Style('wfs_style')
#        self.wfs_style.rules.append(mapping.Rule('catch-all', wfs_sym))
#        self.wfs_style.rules.append(mapping.Rule('catch-all', wfs_text))
#
#        # Diagonal to test Jean-Denis's auto angle patch
#        line = spatial.LineString(wkt='LINESTRING (-1.0 50.5, -0.5 51.2, 0.3 50.9)')
#        atts = ('A Line',)
#        source = memory.MemoryFeatureSource('inline', [['FNAME', 'string']], self.srs)
#        source.add(line, atts)
#        self.memline_store = memory.MemoryFeatureStore()
#        self.memline_store['inline'] = source
#        self.memline_layer = mapping.Layer(self.memline_store, 'inline')
#
#        # --------------------------------------------------------------------
#        # Map renderer
#        self.mapper = rendering.MapRenderer('MAPSERVER', fontset=FONTSET,
#                                          incoming='/var/www-tmp/')

    # tests

class RenderingTest(MapServerRendererTestCase):

    def testGetEmptyMap(self):
        """make a map image"""
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-2, 52),
                            spatial.Point(2, 50))
        size = [200, 200]
        image = self.mapper.render([], view, 'image/jpeg', size,
                                   bgcolor='#8080FF')
        assert image.size == size
        assert image.mimetype == 'image/jpeg'
        f = open('testGetEmptyMap.jpg', 'wb')
        f.write(image.getdata())
        f.close()

    def testLineLayerMap(self):
        """make a map image from a single layer"""
        # styling
        s = mapping.Stroke()
        s['stroke'] = '#80FF80'
        s['stroke-width'] = 5
        symb = mapping.LineSymbolizer(s)
        style = mapping.Style('style')
        style.rules.append(mapping.Rule('catch-all', symb))

        # map canvas
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 51))
        size = [200, 200]
        
        image = self.mapper.render([(self.l_layer, style)], view, 'image/gif',
                                   size)
        
        assert image.size == size
        assert image.mimetype == 'image/gif'
        f = open('testLineLayerMap.gif', 'wb')
        f.write(image.getdata())
        f.close()

    def testStyledPolygonLayerMap(self):
        """make a map image from a single layer using a defined style"""
        # the canvas
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
        
        image = self.mapper.render([(self.p_layer, self.p_style)], view, 
                                   'image/png', size, transparent=1)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testStyledPolygonLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testExtrinsicallyStyledPolygonLayerMap(self):
        """make a map image from a single layer using an extrinsic style"""
        # the canvas
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
        
        image = self.mapper.render([(self.p_layer, self.pe_style)], view, 
                                   'image/png', size, transparent=1)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testExtrinsicallyStyledPolygonLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testInlinePolygonLayerMap(self):
        """make a map image from inline data using a defined style"""
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1.25, 51.5),
                            spatial.Point(0.75, 49.5))
        size = [200, 200]
        image = self.mapper.render([(self.w_layer, self.p_style)], view,
                                   'image/png', size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testInlinePolygonLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testInlineMarkPointLayerMap(self):
        """make a map image from a point layer using a defined style"""
        # --------------------------------------------------------------------
        # inline data layer
        geoms = [spatial.Point(0.3, 51.5), spatial.Point(-0.7, 50.3)]
        atts = [('A Point',), ('B Point',)]
        source = memory.MemoryFeatureSource('inline', [('FNAME', 'string')],
                                            self.srs)
        for g, a in zip(geoms, atts):
            source.add(g, a)
        store = memory.MemoryFeatureStore()
        store['inline'] = source
        layer = mapping.Layer(store, 'inline')
        square = mapping.Mark('square')
        fn = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
        sym = mapping.PointSymbolizer(12, mark=square)
        #style = mapping.ExtrinsicStyle('style')
        style = mapping.Style('style')
        style.rules.append(mapping.Rule('c', fn))
        style.rules.append(mapping.Rule('c', sym))
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
        image = self.mapper.render([(layer, style)], view,
                                   'image/png', size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testInlineMarkPointLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testWMSRasterLayerMap(self):
        """make a map image from a single layer using default styles"""
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, ul=spatial.Point(-2, 54),
                                         lr=spatial.Point(2, 50))
        size = [200, 200]
        image = self.mapper.render([(self.wms_layer, None)], view,
                                   'image/jpeg', size)
                                   
        assert image.size == size
        assert image.mimetype == 'image/jpeg'
        f = open('testWMSRasterLayerMap.jpg', 'wb')
        f.write(image.getdata())
        f.close()

    def testWMSRasterLayerMapPNG(self):
        """make a map image from a single layer using default styles"""
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, ul=spatial.Point(-2, 54),
                                         lr=spatial.Point(2, 50))
        size = [200, 200]
        self.wms_layer.wms_format = 'image/jpeg'
        image = self.mapper.render([(self.wms_layer, None)], view,
                                   'image/png; mode=RGB', size)
                                   
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testWMSRasterLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testRasterLayerMap(self):
        """make a map image from a single layer using default styles"""
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, ul=spatial.Point(-2, 54),
                                         lr=spatial.Point(2, 50))
        size = [200, 200]
        image = self.mapper.render([(self.r_layer, None)], view,
                                   'image/jpeg', size, transparent=1)
                                   
        assert image.size == size
        assert image.mimetype == 'image/jpeg'
        f = open('testRasterLayerMap.jpg', 'wb')
        f.write(image.getdata())
        f.close()

    def testDefaultMarkPointLayerMap(self):
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
        image = self.mapper.render([(self.x_layer, None)], view,
                                   'image/png', size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testDefaultMarkPointLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testWFSPointLayerMap(self):
        #srs = SpatialReference(epsg=4326)
        srs = SpatialReference(epsg=26910)
        #view = mapping.View(srs, spatial.Point(-130, 52),
        #                    spatial.Point(-90, 32))
        view = mapping.View(srs, spatial.Point(300000, 5600000),
                            spatial.Point(700000, 5200000))
        size = [400, 200]
        self.wms_layer.ows_format = 'image/jpeg'
        image = self.mapper.render([(self.wms_layer, None),
                                    (self.wfs_layer, self.wfs_style)],
                                   view, 'image/png; mode=RGB; transparent=ON',
                                   size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testWFSPointLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testMarkPointLayerMap(self):
        """make a map image from a point layer using a defined style"""
        circle = mapping.Mark('square')
        fn = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
        sym = mapping.PointSymbolizer(32, mark=circle)
        style = mapping.Style('style')
        style.rules.append(mapping.Rule('c', fn))
        style.rules.append(mapping.Rule('c', sym))
        
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52),
                            spatial.Point(1, 50))
        size = [200, 200]
    
        image = self.mapper.render([(self.x_layer, style)], view,
                                   'image/png', size, transparent=0)
                                   
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testMarkPointLayerMap.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testAnnotationPoint(self):
        """make a map image from an annotation layer using a defined style"""
        fn = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
        style = mapping.Style('style')
        style.rules.append(mapping.Rule('c', fn))
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1, 52), spatial.Point(1, 50))
        size = [200, 200]
        image = self.mapper.render([(self.x_layer, style)], view,
                                   'image/png', size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testAnnotationPoint.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testGraphicPointLayerMap(self):
        """make a map image from a point layer using a defined style"""
        house = mapping.Image()
        f = open(os.path.join(TESTPATH, 'home.png'), 'rb')
        house.data.write(f.read())
        f.close()
        symb = mapping.PointSymbolizer(32, graphic=house)
        x_style = mapping.Style('x_style')
        x_style.rules.append(mapping.Rule('c', symb))
        
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1.5, 53.0),
                            spatial.Point(1.5, 50.0))
        size = [200, 200]
        layering = [(self.r_layer, None), (self.x_layer, x_style)]
        image = self.mapper.render(layering, view, 'image/jpeg',
                                   size, transparent=1)
                                   
        assert image.size == size
        assert image.mimetype == 'image/jpeg'
        f = open('testGraphicPointLayerMap.jpg', 'wb')
        f.write(image.getdata())
        f.close()

    def testRuleScales(self):
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1.25, 51.5),
                            spatial.Point(0.75, 49.5), 'METERS')
        size = [200, 200]
        # Set rule scales
        self.pe_style.rules[2].maxscaledenominator = 20.0
        self.pe_style.rules[3].maxscaledenominator = 20.0
        image = self.mapper.render([(self.w_layer, self.pe_style)], view,
                                   'image/png', size, transparent=0)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testRuleScales.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testLabelAutoAngle(self):
        """testing issue 83"""
        # styling
        s = mapping.Stroke()
        s['stroke'] = '#80FF80'
        s['stroke-width'] = 5
        symb = mapping.LineSymbolizer(s)
        style = mapping.Style('style')
        style.rules.append(mapping.Rule('catch-all', symb))
        text = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
        style.rules.append(mapping.Rule('label-all', text))

        # map canvas
        srs = SpatialReference(epsg=4326)
        view = mapping.View(srs, spatial.Point(-1.2, 51.5),
                            spatial.Point(0.8, 50.5))
        size = [200, 200]
        
        image = self.mapper.render([(self.memline_layer, style)], view,
                                   'image/png', size)
        f = open('testLabelAutoAngle.png', 'wb')
        f.write(image.getdata())
        f.close()


class MapServerTestCase(unittest.TestCase):

    def setUp(self):
        self.mo = mapscript.mapObj()
        li = self.mo.insertLayer(mapscript.layerObj())
        self.lo = self.mo.getLayer(li)
        ci = self.lo.insertClass(mapscript.classObj())
        self.co = self.lo.getClass(ci)
        si = self.co.insertStyle(mapscript.styleObj())
        self.so = self.co.getStyle(si)

class StylingTestCase(MapServerTestCase):
    pass
   

class ReprojectRenderingTest(MapServerRendererTestCase):

    def testProjectMemline(self):
        """make a map image"""
        p = "+proj=aea +ellps=clrk66 +lat_1=40.8333333 +lat_2=64.1666666 +lat_0=25.0 +lon_0=0.0 +x_0=0.0 +y_0=0.0 +units=m"
        srs = SpatialReference(p)
        view = mapping.View(srs,
                            spatial.Point(-140746.27419591, 3192511.533156885),
                            spatial.Point(140746.27419591, 2736712.303835521))
        size = [200, 200]
        image = self.mapper.render([(self.memline_layer, None),
                                    (self.x_layer, None)], view,
                                   'image/png', size, bgcolor='#8080FF')
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testProjectMemline.png', 'wb')
        f.write(image.getdata())
        f.close()


class DrawSymbolizerTestCase(unittest.TestCase):

    def setUp(self):
        self.mapper = rendering.MapRenderer('MAPSERVER', fontset=FONTSET)

    def testDrawMarkerPointSymbolizer(self):
        """draw a point symbolizer image"""
        circle = mapping.Mark('circle')
        sym = mapping.PointSymbolizer(8, mark=circle)
        size = [16, 16]
        image = self.mapper.draw(sym, 'image/png', size)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testDrawMarkerPointSymbolizer.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testCrazyMarkSymbolizer(self):
        circle = mapping.Mark('circle')
        sym = mapping.PointSymbolizer(12, mark=circle)
        sym.mark.stroke['stroke-width'] = 3
        sym.mark.fill['fill'] = '#FF0000'
        sym.mark.fill['fill-opacity'] = 0.5
        size = [16, 16]
        image = self.mapper.draw(sym, 'image/png', size)
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testCrazyMarkSymbolizer.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testDrawLineSymbolizer(self):
        """draw a line symbolizer image"""
        s = mapping.Stroke()
        s['stroke'] = '#80FF80'
        s['stroke-width'] = 5
        sym = mapping.LineSymbolizer(s)
        size = [48, 48]
        image = self.mapper.draw(sym, 'image/png', size)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testDrawLineSymbolizer.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testDrawPolygonSymbolizer(self):
        """draw a polygon symbolizer image"""
        sym = mapping.PolygonSymbolizer(fill={'fill': '#FF8080'})
        size = [48, 48]
        image = self.mapper.draw(sym, 'image/png', size)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testDrawPolygonSymbolizer.png', 'wb')
        f.write(image.getdata())
        f.close()

    def testDrawTextSymbolizer(self):
        """draw a polygon symbolizer image"""
        text = mapping.TextSymbolizer('FNAME', {'font-family': 'VeraBd'})
        size = [48, 48]
        image = self.mapper.draw(text, 'image/png', size)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
        f = open('testDrawTextSymbolizer.png', 'wb')
        f.write(image.getdata())
        f.close()


import Image

class ImageModeTestCase(unittest.TestCase):

    def setUp(self):
        self.mapper = rendering.MapRenderer('MAPSERVER', fontset=FONTSET)

    def testDrawPNG24MarkerPointSymbolizer(self):
        circle = mapping.Mark('circle')
        sym = mapping.PointSymbolizer(8, mark=circle)
        size = [16, 16]
        image = self.mapper.draw(sym, 'image/png; mode=RGB; interlace=OFF', size)
        
        assert image.size == size
        assert image.mimetype == 'image/png'
    
        f = open('testDrawPNG24.png', 'wb')
        f.write(image.getdata())
        f.close()

        im = Image.open('testDrawPNG24.png')
        self.assert_(im.size == (16, 16), im.size)

    def testDrawBogusMarkerPointSymbolizer(self):
        """draw a point symbolizer image"""
        circle = mapping.Mark('circle')
        sym = mapping.PointSymbolizer(8, mark=circle)
        size = [16, 16]
        self.assertRaises(Exception, self.mapper.draw,
                          sym, 'image/gif; mode=RGB', size)
        

# ===========================================================================
if __name__ == '__main__':
    unittest.main()

