# $Id: timeshapes.py,v 1.1 2005/01/27 04:53:51 sgillies Exp $

# =============================================================================
# Python Cartographic Library. Copyright (C) 2004 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================
import os
import timeit
import unittest
import mapscript

from framework import cartography, msbridge
from cartography.spatial import Point, LineString, LinearRing, Polygon

point = Point(5.02, 10.5)

# =========================================================================
# Test 1A: build up a shapeObj using the mapscript API

print "Test 1A: mapscripting shapeObj construction"

s = """\
so = mapscript.shapeObj(mapscript.MS_SHAPE_POINT)
lo = mapscript.lineObj()
po = mapscript.pointObj(point.x, point.y)
lo.add(po)
so.add(lo)
so.setBounds()
"""

t = timeit.Timer(stmt=s, setup='from __main__ import mapscript, point')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# =========================================================================
# Test 1B: use shapeObj constructor from msbridge

print "Test 1B: shapeObj constructor from msbridge"

s = """\
so = msbridge.shapeObj(0, point)
"""

t = timeit.Timer(stmt=s, setup='from __main__ import msbridge, point')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


points = [Point(-5,-5), Point(0,0), Point(5,5)]
line = LineString(points)

# =========================================================================
# Test 2A: build up a LINE shapeObj using the mapscript API

print "Test 2A: mapscripting N=3 Line"

s = """\
po = mapscript.pointObj()
so = mapscript.shapeObj(mapscript.MS_SHAPE_LINE)
lo = mapscript.lineObj()
for p in line:
    po.setXY(p.x, p.y)
    lo.add(po)
so.add(lo)
so.setBounds()
"""

t = timeit.Timer(stmt=s, setup='from __main__ import mapscript, line')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# =========================================================================
# Test 2B: use shapeObj constructor from msbridge

print "Test 2B: msbridge N=3 Line"

s = """\
so = msbridge.shapeObj(1, line)
"""

t = timeit.Timer(stmt=s, setup='from __main__ import msbridge, line')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


points = [Point(-5,-5), Point(5,5), Point(5,-5), Point(-5,-5)]
polygon = Polygon([LinearRing(points)])

# =========================================================================
# Test 3A: build up a POLYGON shapeObj using the mapscript API

print "Test 3A: mapscript N=4 (single ring) Polygon"

s = """\
po = mapscript.pointObj()
so = mapscript.shapeObj(mapscript.MS_SHAPE_POLYGON)
for ring in polygon:
    lo = mapscript.lineObj()
    for p in ring:
        po.setXY(p.x, p.y)
        lo.add(po)
    so.add(lo)
so.setBounds()
"""

t = timeit.Timer(stmt=s, setup='from __main__ import mapscript, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# =========================================================================
# Test 3B: use POLYGON shapeObj constructor from msbridge

print "Test 3B: msbridge N=4 (single ring) Polygon"

s = """\
so = msbridge.shapeObj(2, polygon)
"""

t = timeit.Timer(stmt=s, setup='from __main__ import msbridge, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# N=9 polygon

points = [Point(-5,-5), Point(-2.5,-2.5), Point(0,0), Point(2.5,2.5),
          Point(5,5), Point(5,0), Point(5,-5), Point(0,-5), Point(-5,-5)]
polygon = Polygon([LinearRing(points)])

# =========================================================================
# Test 4A: build up a POLYGON shapeObj using the mapscript API

print "Test 4A: mapscripting N=9 (single ring) Polygon"

s = """\
po = mapscript.pointObj()
so = mapscript.shapeObj(mapscript.MS_SHAPE_POLYGON)
for ring in polygon:
    lo = mapscript.lineObj()
    for p in ring:
        po.setXY(p.x, p.y)
        lo.add(po)
    so.add(lo)
so.setBounds()
"""

t = timeit.Timer(stmt=s, setup='from __main__ import mapscript, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# =========================================================================
# Test 4B: use POLYGON shapeObj constructor from msbridge

print "Test 4B: msbridge N=9 (single ring) Polygon"

s = """\
so = msbridge.shapeObj(2, polygon)
"""

t = timeit.Timer(stmt=s, setup='from __main__ import msbridge, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# N=13 polygon

points = [Point(-5,-5), Point(-2.5,-2.5), Point(0,0), Point(2.5,2.5),
          Point(5,5), Point(5,2.5), Point(5,0), Point(5,-2.5), 
          Point(5,-5), Point(2.5,-5), Point(0,-5), Point(-2.5,-5),
          Point(-5,-5)]
polygon = Polygon([LinearRing(points)])

# =========================================================================
# Test 5A: build up a POLYGON shapeObj using the mapscript API

print "Test 5A: mapscripting N=13 (single ring) Polygon"

s = """\
po = mapscript.pointObj()
so = mapscript.shapeObj(mapscript.MS_SHAPE_POLYGON)
for ring in polygon:
    lo = mapscript.lineObj()
    for p in ring:
        po.setXY(p.x, p.y)
        lo.add(po)
    so.add(lo)
so.setBounds()
"""

t = timeit.Timer(stmt=s, setup='from __main__ import mapscript, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


# =========================================================================
# Test 5B: use POLYGON shapeObj constructor from msbridge

print "Test 5B: msbridge N=13 (single ring) Polygon"

s = """\
so = msbridge.shapeObj(2, polygon)
"""

t = timeit.Timer(stmt=s, setup='from __main__ import msbridge, polygon')
print "%.2f usec/pass" % (1000000 * t.timeit(number=1000)/1000)


