# =============================================================================
# Python Cartographic Library. Copyright (C) 2006 Sean C. Gillies
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA
#
# Contact email: sgillies@frii.com
# =============================================================================

import os, sys
from setuptools import setup, Extension

# ---------------------------------------------------------------------------
# The MapServer engine
# ---------------------------------------------------------------------------
# Specify absolute path to MapServer source 
ms_home = '/usr/local/src/mapserver-4.8.4'    # <-- edit if necessary
if not os.path.exists(ms_home):
    print >>sys.stderr, "ms_home is not set correctly in PCL-Mapserver/tsetup.py"
    print >>sys.stderr, "it is currently set to", ms_home, "which does not exist!"
    sys.exit( -1 )

# Generate msbridge.py and msbridge_wrap.c from SWIG interface
mb_path = 'cartography/engine/mapserver'
pcl_home = os.getcwd()

# Try to generate msbridge wrapper if not present
if not os.path.exists(os.path.join(mb_path, 'msbridge_wrap.c')):
    os.chdir(mb_path)
    os.system('swig -python -modern -keyword -o msbridge_wrap.c msbridge.i')
    os.chdir(pcl_home)

# Define msbridge extension module
mb_ext = Extension('cartography.engine.mapserver._msbridge',
            sources=[os.path.join(mb_path, 'msbridge_wrap.c'),
            os.path.join(mb_path, 'mapserverbridge.c')],
            include_dirs=[ms_home, mb_path],
            libraries=['geos_c'])

# ---------------------------------------------------------------------------
# Execute setup
# ---------------------------------------------------------------------------
setup(name          = 'PCL-MapServer',
      version       = '0.11.0',
      description   = 'MapServer-based rendering engine',
      author        = 'Sean Gillies',
      author_email  = 'sgillies@frii.com',
      url           = 'http://www.gispython.org',
      namespace_packages    = ['cartography', 'cartography.engine'],
      packages              = ['cartography', 'cartography.engine',
                               'cartography.engine.mapserver'],
      ext_modules           = [mb_ext],
      install_requires = ['PCL-Core'],
)

